@@ -1,10 +1,12 @@
 import pytest
+
+from indy_common.auth import Authoriser, generate_auth_map
 from plenum.common.constants import STEWARD, TRUSTEE
 
-from indy_common.constants import TRUST_ANCHOR, TGB
+from indy_common.constants import TRUST_ANCHOR
 
 
-@pytest.fixture(scope='function', params=[STEWARD, TRUSTEE, TRUST_ANCHOR, TGB, None])
+@pytest.fixture(scope='function', params=[STEWARD, TRUSTEE, TRUST_ANCHOR, None])
 def role(request):
     return request.param
 
@@ -17,3 +19,8 @@ def is_owner(request):
 @pytest.fixture(scope='function', params=[None, "value1"])
 def old_values(request):
     return request.param
+
+
+@pytest.fixture(scope='module')
+def initialized_auth_map():
+    Authoriser.auth_map = generate_auth_map(Authoriser.ValidRoles, False)
