@@ -1,7 +1,7 @@
 from plenum.common.constants import TRUSTEE, STEWARD, VERKEY
 
 from indy_common.auth import Authoriser
-from indy_common.constants import ROLE, NYM, TGB, TRUST_ANCHOR
+from indy_common.constants import ROLE, NYM, TRUST_ANCHOR
 
 
 def test_make_trustee(role, is_owner):
@@ -14,16 +14,6 @@ def test_make_trustee(role, is_owner):
                                                isActorOwnerOfSubject=is_owner)[0]
 
 
-def test_make_tgb(role, is_owner):
-    authorized = (role == TRUSTEE)
-    assert authorized == Authoriser.authorised(typ=NYM,
-                                               actorRole=role,
-                                               field=ROLE,
-                                               oldVal=None,
-                                               newVal=TGB,
-                                               isActorOwnerOfSubject=is_owner)[0]
-
-
 def test_make_steward(role, is_owner):
     authorized = (role == TRUSTEE)
     assert authorized == Authoriser.authorised(typ=NYM,
@@ -54,16 +44,6 @@ def test_remove_trustee(role, is_owner):
                                                isActorOwnerOfSubject=is_owner)[0]
 
 
-def test_remove_tgb(role, is_owner):
-    authorized = (role == TRUSTEE)
-    assert authorized == Authoriser.authorised(typ=NYM,
-                                               actorRole=role,
-                                               field=ROLE,
-                                               oldVal=TGB,
-                                               newVal=None,
-                                               isActorOwnerOfSubject=is_owner)[0]
-
-
 def test_remove_steward(role, is_owner):
     authorized = (role == TRUSTEE)
     assert authorized == Authoriser.authorised(typ=NYM,
