@@ -2,13 +2,11 @@ import dateutil.parser
 
 from plenum.common.exceptions import InvalidClientRequest, \
     UnauthorizedClientRequest
-from plenum.common.messages.node_messages import Reply
-from plenum.common.txn_util import reqToTxn
 from plenum.common.types import f
 from plenum.server.req_handler import RequestHandler
 from plenum.common.constants import TXN_TYPE, DATA
 from indy_common.auth import Authoriser
-from indy_common.constants import ACTION, POOL_RESTART, DATETIME, VALIDATOR_INFO
+from indy_common.constants import ACTION, POOL_RESTART, VALIDATOR_INFO
 from indy_common.roles import Roles
 from indy_common.types import Request
 from indy_node.persistence.idr_cache import IdrCache
@@ -71,11 +69,15 @@ class ActionReqHandler(RequestHandler):
                      .format(req.reqId, req.txn_type))
         try:
             if req.txn_type == POOL_RESTART:
-                self.restarter.handleActionTxn(req)
+                self.restarter.handleRestartRequest(req)
                 result = self._generate_action_result(req)
             elif req.txn_type == VALIDATOR_INFO:
                 result = self._generate_action_result(req)
                 result[DATA] = self.info_tool.info
+                result[DATA].update(self.info_tool.memory_profiler)
+                result[DATA].update(self.info_tool._generate_software_info())
+                result[DATA].update(self.info_tool.extractions)
+                result[DATA].update(self.info_tool.node_disk_size)
             else:
                 raise InvalidClientRequest(
                     "{} is not type of action transaction"
