@@ -1,22 +1,22 @@
 import os
-from datetime import datetime, timedelta
+import asyncio
+from datetime import datetime
 from functools import partial
-from typing import Tuple, Union, Optional, Callable, Dict
+from typing import Union, Optional, Callable, Dict
 
 import dateutil.parser
 import dateutil.tz
 
 from indy_node.server.node_maintainer import NodeMaintainer, \
     NodeControlToolMessage
+from plenum.common.txn_util import is_forced, get_seq_no, get_type, get_payload_data, get_req_id, get_from
 from stp_core.common.log import getlogger
-from plenum.common.constants import TXN_TYPE, VERSION, DATA, IDENTIFIER
-from plenum.common.types import f
+from plenum.common.constants import VERSION
 from indy_common.constants import ACTION, POOL_UPGRADE, START, SCHEDULE, \
-    CANCEL, JUSTIFICATION, TIMEOUT, REINSTALL, NODE_UPGRADE, FORCE, \
-    UPGRADE_MESSAGE
+    CANCEL, JUSTIFICATION, TIMEOUT, REINSTALL, NODE_UPGRADE, \
+    UPGRADE_MESSAGE, PACKAGE, APP_NAME
 from indy_node.server.upgrade_log import UpgradeLog
-from ledger.util import F
-import asyncio
+from indy_node.utils.node_control_utils import NodeControlUtil
 
 logger = getlogger()
 
@@ -31,7 +31,11 @@ class Upgrader(NodeMaintainer):
                          actionFailedCallback, action_start_callback)
 
     @staticmethod
-    def getVersion():
+    def getVersion(pkg: str = None):
+        if pkg and pkg != APP_NAME:
+            ver, _ = NodeControlUtil.curr_pkt_info(pkg)
+            return ver
+
         from indy_node.__metadata__ import __version__
         return __version__
 
@@ -42,10 +46,10 @@ class Upgrader(NodeMaintainer):
 
     @staticmethod
     def get_action_id(txn):
-        seq_no = txn.get(F.seqNo.name, '')
-        if txn.get(FORCE, None):
+        seq_no = get_seq_no(txn) or ''
+        if is_forced(txn):
             seq_no = ''
-        return '{}{}'.format(txn[f.REQ_ID.nm], seq_no)
+        return '{}{}'.format(get_req_id(txn), seq_no)
 
     @staticmethod
     def compareVersions(verA: str, verB: str) -> int:
@@ -78,38 +82,36 @@ class Upgrader(NodeMaintainer):
 
     def _is_action_started(self):
         if not self.lastActionEventInfo:
-            logger.debug('Node {} has no upgrade events'
-                         .format(self.nodeName))
+            logger.debug('Node {} has no upgrade events'.format(self.nodeName))
             return False
 
-        (event_type, when, version, upgrade_id) = self.lastActionEventInfo
+        (event_type, when, version, upgrade_id, pkg_name) = self.lastActionEventInfo
 
         if event_type != UpgradeLog.STARTED:
             logger.debug(
-                'Upgrade for node {} was not scheduled. Last event is {}:{}:{}:{}'.format(
-                    self.nodeName, event_type, when, version, upgrade_id))
+                'Upgrade for node {} was not scheduled. Last event is {}:{}:{}:{}:{}'.format(
+                    self.nodeName, event_type, when, version, upgrade_id, pkg_name))
             return False
 
         return True
 
     def _update_action_log_for_started_action(self):
-        (event_type, when, version, upgrade_id) = self.lastActionEventInfo
+        (event_type, when, version, upgrade_id, pkg_name) = self.lastActionEventInfo
         self._should_notify_about_upgrade = True
         if not self.didLastExecutedUpgradeSucceeded:
-            self._actionLog.appendFailed(when, version, upgrade_id)
+            self._actionLog.appendFailed(when, version, upgrade_id, pkg_name)
             self._action_failed(version=version,
                                 scheduled_on=when,
                                 upgrade_id=upgrade_id,
                                 external_reason=True)
             return
 
-        self._actionLog.appendSucceeded(when, version, upgrade_id)
-        logger.info("Node '{}' successfully upgraded to version {}"
-                    .format(self.nodeName, version))
+        self._actionLog.appendSucceeded(when, version, upgrade_id, pkg_name)
+        logger.info("Node '{}' successfully upgraded to version {}".format(self.nodeName, version))
         self._notifier.sendMessageUponNodeUpgradeComplete(
-            "Upgrade of node '{}' to version {} scheduled on {} "
+            "Upgrade of packet {} on node '{}' to version {} scheduled on {} "
             " with upgrade_id {} completed successfully"
-            .format(self.nodeName, version, when, upgrade_id))
+            .format(pkg_name, self.nodeName, version, when, upgrade_id))
 
     def should_notify_about_upgrade_result(self):
         # do not rely on NODE_UPGRADE txn in config ledger, since in
@@ -126,7 +128,7 @@ class Upgrader(NodeMaintainer):
 
     def get_last_node_upgrade_txn(self, start_no: int = None):
         return self.get_upgrade_txn(
-            lambda txn: txn[TXN_TYPE] == NODE_UPGRADE and txn[IDENTIFIER] == self.nodeId,
+            lambda txn: get_type(txn) == NODE_UPGRADE and get_from(txn) == self.nodeId,
             start_no=start_no,
             reverse=True)
 
@@ -167,24 +169,24 @@ class Upgrader(NodeMaintainer):
         logger.debug(
             '{} processing config ledger for any upgrades'.format(self))
         last_pool_upgrade_txn_start = self.get_upgrade_txn(
-            lambda txn: txn[TXN_TYPE] == POOL_UPGRADE and txn[ACTION] == START, reverse=True)
+            lambda txn: get_type(txn) == POOL_UPGRADE and get_payload_data(txn)[ACTION] == START, reverse=True)
         if last_pool_upgrade_txn_start:
             logger.info('{} found upgrade START txn {}'.format(
                 self, last_pool_upgrade_txn_start))
-            last_pool_upgrade_txn_seq_no = last_pool_upgrade_txn_start[F.seqNo.name]
+            last_pool_upgrade_txn_seq_no = get_seq_no(last_pool_upgrade_txn_start)
 
             # searching for CANCEL for this upgrade submitted after START txn
             last_pool_upgrade_txn_cancel = self.get_upgrade_txn(
                 lambda txn:
-                txn[TXN_TYPE] == POOL_UPGRADE and txn[ACTION] == CANCEL and
-                txn[VERSION] == last_pool_upgrade_txn_start[VERSION],
+                get_type(txn) == POOL_UPGRADE and get_payload_data(txn)[ACTION] == CANCEL and
+                get_payload_data(txn)[VERSION] == get_payload_data(last_pool_upgrade_txn_start)[VERSION],
                 start_no=last_pool_upgrade_txn_seq_no + 1)
             if last_pool_upgrade_txn_cancel:
                 logger.info('{} found upgrade CANCEL txn {}'.format(
                     self, last_pool_upgrade_txn_cancel))
                 return
 
-            self.handleActionTxn(last_pool_upgrade_txn_start)
+            self.handleUpgradeTxn(last_pool_upgrade_txn_start)
 
     @property
     def didLastExecutedUpgradeSucceeded(self) -> bool:
@@ -197,12 +199,12 @@ class Upgrader(NodeMaintainer):
         """
         lastEventInfo = self.lastActionEventInfo
         if lastEventInfo:
-            currentVersion = self.getVersion()
+            currentVersion = self.getVersion(lastEventInfo[4])
             scheduledVersion = lastEventInfo[2]
             return self.compareVersions(currentVersion, scheduledVersion) == 0
         return False
 
-    def handleActionTxn(self, txn) -> None:
+    def handleUpgradeTxn(self, txn) -> None:
         """
         Handles transaction of type POOL_UPGRADE
         Can schedule or cancel upgrade to a newer
@@ -210,24 +212,22 @@ class Upgrader(NodeMaintainer):
 
         :param txn:
         """
-        FINALIZING_EVENT_TYPES = [
-            UpgradeLog.SUCCEEDED, UpgradeLog.FAILED]
+        FINALIZING_EVENT_TYPES = [UpgradeLog.SUCCEEDED, UpgradeLog.FAILED]
 
-        if txn[TXN_TYPE] != POOL_UPGRADE:
+        if get_type(txn) != POOL_UPGRADE:
             return
 
-        logger.info("Node '{}' handles upgrade txn {}".format(
-            self.nodeName, txn))
-        action = txn[ACTION]
-        version = txn[VERSION]
-        justification = txn.get(JUSTIFICATION)
-        reinstall = txn.get(REINSTALL, False)
-        currentVersion = self.getVersion()
+        logger.info("Node '{}' handles upgrade txn {}".format(self.nodeName, txn))
+        txn_data = get_payload_data(txn)
+        action = txn_data[ACTION]
+        version = txn_data[VERSION]
+        justification = txn_data.get(JUSTIFICATION)
+        pkg_name = txn_data.get(PACKAGE, self.config.UPGRADE_ENTRY)
         upgrade_id = self.get_action_id(txn)
 
         if action == START:
             # forced txn could have partial schedule list
-            if self.nodeId not in txn[SCHEDULE]:
+            if self.nodeId not in txn_data[SCHEDULE]:
                 logger.info("Node '{}' disregards upgrade txn {}".format(
                     self.nodeName, txn))
                 return
@@ -240,17 +240,13 @@ class Upgrader(NodeMaintainer):
                         self.nodeName, upgrade_id, last_event))
                 return
 
-            when = txn[SCHEDULE][self.nodeId]
-            failTimeout = txn.get(TIMEOUT, self.defaultActionTimeout)
-
-            if not self.is_version_upgradable(
-                    currentVersion, version, reinstall):
-                return
+            when = txn_data[SCHEDULE][self.nodeId]
+            failTimeout = txn_data.get(TIMEOUT, self.defaultActionTimeout)
 
             if self.scheduledAction:
                 if isinstance(when, str):
                     when = dateutil.parser.parse(when)
-                if self.scheduledAction == (version, when, upgrade_id):
+                if self.scheduledAction == (version, when, upgrade_id, pkg_name):
                     logger.debug("Node {} already scheduled upgrade to version '{}' ".format(
                         self.nodeName, version))
                     return
@@ -260,11 +256,9 @@ class Upgrader(NodeMaintainer):
                             self.nodeName, version))
                     self._cancelScheduledUpgrade(justification)
 
-            logger.info("Node '{}' schedules upgrade to {}".format(
-                self.nodeName, version))
+            logger.info("Node '{}' schedules upgrade to {}".format(self.nodeName, version))
 
-            self._scheduleUpgrade(
-                version, when, failTimeout, upgrade_id)
+            self._scheduleUpgrade(version, when, failTimeout, upgrade_id, pkg_name)
             return
 
         if action == CANCEL:
@@ -282,32 +276,32 @@ class Upgrader(NodeMaintainer):
                          version,
                          when: Union[datetime, str],
                          failTimeout,
-                         upgrade_id) -> None:
+                         upgrade_id,
+                         pkg_name) -> None:
         """
         Schedules node upgrade to a newer version
 
         :param version: version to upgrade to
         :param when: upgrade time
         :param upgrade_id: upgrade identifier (req_id+seq_no) of a txn that started the upgrade
+        :param pkg_name: packet to be upgraded
         """
         assert isinstance(when, (str, datetime))
-        logger.info("{}'s upgrader processing upgrade for version {}"
-                    .format(self, version))
+        logger.info("{}'s upgrader processing upgrade for version {}={}".format(self, pkg_name, version))
         if isinstance(when, str):
             when = dateutil.parser.parse(when)
         now = datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc())
 
         self._notifier.sendMessageUponNodeUpgradeScheduled(
-            "Upgrade of node '{}' to version {} has been scheduled on {}".format(
-                self.nodeName, version, when))
-        self._actionLog.appendScheduled(when, version, upgrade_id)
+            "Upgrade of packet {} on node '{}' to version {} has been scheduled on {}".format(
+                pkg_name, self.nodeName, version, when))
+        self._actionLog.appendScheduled(when, version, upgrade_id, pkg_name)
 
-        callAgent = partial(self._callUpgradeAgent, when,
-                            version, failTimeout, upgrade_id)
+        callAgent = partial(self._callUpgradeAgent, when, version, failTimeout, upgrade_id, pkg_name)
         delay = 0
         if now < when:
             delay = (when - now).total_seconds()
-        self.scheduledAction = (version, when, upgrade_id)
+        self.scheduledAction = (version, when, upgrade_id, pkg_name)
         self._schedule(callAgent, delay)
 
     def _cancelScheduledUpgrade(self, justification=None) -> None:
@@ -325,28 +319,28 @@ class Upgrader(NodeMaintainer):
                 why_prefix = ", "
                 why = "cancellation reason not specified"
 
-            (version, when, upgrade_id) = self.scheduledAction
+            (version, when, upgrade_id, pkg_name) = self.scheduledAction
             logger.info("Cancelling upgrade {upgrade_id}"
                         " of node {node}"
+                        " of packet {packet}"
                         " to version {version}"
                         " scheduled on {when}"
                         "{why_prefix}{why}"
                         .format(upgrade_id=upgrade_id,
                                 node=self.nodeName,
+                                packet=pkg_name,
                                 version=version,
                                 when=when,
                                 why_prefix=why_prefix,
                                 why=why))
 
             self._unscheduleAction()
-            self._actionLog.appendCancelled(when, version, upgrade_id)
+            self._actionLog.appendCancelled(when, version, upgrade_id, pkg_name)
             self._notifier.sendMessageUponPoolUpgradeCancel(
-                "Upgrade of node '{}' to version {} "
-                "has been cancelled due to {}".format(
-                    self.nodeName, version, why))
+                "Upgrade of packet {} on node '{}' to version {} "
+                "has been cancelled due to {}".format(pkg_name, self.nodeName, version, why))
 
-    def _callUpgradeAgent(self, when, version, failTimeout,
-                          upgrade_id) -> None:
+    def _callUpgradeAgent(self, when, version, failTimeout, upgrade_id, pkg_name) -> None:
         """
         Callback which is called when upgrade time come.
         Writes upgrade record to upgrade log and asks
@@ -357,36 +351,33 @@ class Upgrader(NodeMaintainer):
         """
 
         logger.info("{}'s upgrader calling agent for upgrade".format(self))
-        self._actionLog.appendStarted(when, version, upgrade_id)
+        self._actionLog.appendStarted(when, version, upgrade_id, pkg_name)
         self._action_start_callback()
         self.scheduledAction = None
         asyncio.ensure_future(
-            self._sendUpgradeRequest(when, version, upgrade_id, failTimeout))
+            self._sendUpgradeRequest(when, version, upgrade_id, failTimeout, pkg_name))
 
-    async def _sendUpgradeRequest(self, when, version, upgrade_id, failTimeout):
+    async def _sendUpgradeRequest(self, when, version, upgrade_id, failTimeout, pkg_name):
         retryLimit = self.retry_limit
         while retryLimit:
             try:
-                msg = UpgradeMessage(version=version).toJson()
+                msg = UpgradeMessage(version=version, pkg_name=pkg_name).toJson()
                 logger.info("Sending message to control tool: {}".format(msg))
                 await self._open_connection_and_send(msg)
                 break
             except Exception as ex:
-                logger.warning("Failed to communicate to control tool: {}"
-                               .format(ex))
+                logger.warning("Failed to communicate to control tool: {}".format(ex))
                 asyncio.sleep(self.retry_timeout)
                 retryLimit -= 1
         if not retryLimit:
             self._action_failed(version=version,
                                 scheduled_on=when,
                                 upgrade_id=upgrade_id,
-                                reason="problems in communication with "
-                                       "node control service")
+                                reason="problems in communication with node control service")
             self._unscheduleAction()
             self._actionFailedCallback()
         else:
-            logger.info("Waiting {} minutes for upgrade to be performed"
-                        .format(failTimeout))
+            logger.info("Waiting {} minutes for upgrade to be performed".format(failTimeout))
             timesUp = partial(self._declareTimeoutExceeded, when, version, upgrade_id)
             self._schedule(timesUp, self.get_timeout(failTimeout))
 
@@ -471,9 +462,10 @@ class UpgradeMessage(NodeControlToolMessage):
     Data structure that represents request for node update
     """
 
-    def __init__(self, version: str):
+    def __init__(self, version: str, pkg_name: str):
         super().__init__(UPGRADE_MESSAGE)
         self.version = version
+        self.pkg_name = pkg_name
 
     def toJson(self):
         import json
