@@ -2,6 +2,8 @@ import logging
 import warnings
 import pytest
 
+from indy_common.config_helper import NodeConfigHelper
+from indy_node.test.helper import TestNode
 from plenum.test.pool_transactions.helper import sdk_add_new_nym, sdk_pool_refresh, prepare_new_node_data, \
     create_and_start_new_node, prepare_node_request, sdk_sign_and_send_prepared_request
 from stp_core.common.log import Logger
@@ -15,17 +17,10 @@ from indy_common import strict_types
 # typecheck during tests
 strict_types.defaultShouldCheck = True
 
-# noinspection PyUnresolvedReferences
-from indy_client.test.conftest import trustAnchorWallet, \
-    trustAnchor, tdirWithDomainTxns, \
-    stewardWallet, steward, genesisTxns, testClientClass, client_ledger_dir, \
-    addedTrustAnchor, userWalletB, nodeSet, testNodeClass, updatedPoolTxnData, \
-    trusteeData, trusteeWallet, trustee, warnfilters as client_warnfilters
-
 # noinspection PyUnresolvedReferences
 from plenum.test.conftest import tdir, client_tdir, nodeReg, \
     whitelist, concerningLogLevels, logcapture, \
-    tdirWithPoolTxns, tdirWithDomainTxns as PTdirWithDomainTxns, \
+    tdirWithPoolTxns, tdirWithDomainTxns, \
     tdirWithClientPoolTxns, txnPoolNodeSet, \
     poolTxnData, dirName, poolTxnNodeNames, allPluginsPath, tdirWithNodeKeepInited, \
     poolTxnStewardData, poolTxnStewardNames, getValueFromModule, \
@@ -37,10 +32,10 @@ from indy_common.test.conftest import general_conf_tdir, tconf, poolTxnTrusteeNa
     domainTxnOrderedFields, looper, setTestLogLevel, node_config_helper_class, config_helper_class
 
 # noinspection PyUnresolvedReferences
-from plenum.test.conftest import sdk_pool_handle as plenum_pool_handle, sdk_pool_name, sdk_wallet_steward, \
-    sdk_wallet_handle, sdk_wallet_name, sdk_steward_seed, sdk_wallet_client, sdk_wallet_trustee, \
+from plenum.test.conftest import sdk_pool_handle as plenum_pool_handle, sdk_pool_data, sdk_wallet_steward, \
+    sdk_wallet_handle, sdk_wallet_data, sdk_steward_seed, sdk_wallet_client, sdk_wallet_trustee, \
     sdk_trustee_seed, trustee_data, sdk_client_seed, poolTxnClientData, poolTxnClientNames, \
-    sdk_wallet_stewards
+    sdk_wallet_stewards, create_node_and_not_start
 
 Logger.setLogLevel(logging.NOTSET)
 
@@ -51,9 +46,8 @@ def sdk_pool_handle(plenum_pool_handle, nodeSet):
 
 
 @pytest.fixture(scope="session")
-def warnfilters(client_warnfilters):
+def warnfilters():
     def _():
-        client_warnfilters()
         warnings.filterwarnings(
             'ignore',
             category=DeprecationWarning,
@@ -85,7 +79,7 @@ def sdk_node_theta_added(looper,
                                          alias=new_steward_name,
                                          role=STEWARD_STRING)
 
-    sigseed, verkey, bls_key, nodeIp, nodePort, clientIp, clientPort = \
+    sigseed, verkey, bls_key, nodeIp, nodePort, clientIp, clientPort, key_proof = \
         prepare_new_node_data(tconf, tdir, new_node_name,
                               configClass=node_config_helper_class)
 
@@ -100,7 +94,8 @@ def sdk_node_theta_added(looper,
                              nodePort=nodePort,
                              bls_key=bls_key,
                              sigseed=sigseed,
-                             services=[VALIDATOR]))
+                             services=[VALIDATOR],
+                             key_proof=key_proof))
 
     # sending request using 'sdk_' functions
     request_couple = sdk_sign_and_send_prepared_request(looper, new_steward_wallet,
@@ -125,3 +120,42 @@ def sdk_node_theta_added(looper,
 def sdk_wallet_trust_anchor(looper, sdk_pool_handle, sdk_wallet_trustee):
     return sdk_add_new_nym(looper, sdk_pool_handle, sdk_wallet_trustee,
                            alias='TA-1', role='TRUST_ANCHOR')
+
+
+@pytest.fixture(scope="module")
+def sdk_user_wallet_a(nodeSet, sdk_wallet_trust_anchor,
+                      sdk_pool_handle, looper):
+    return sdk_add_new_nym(looper, sdk_pool_handle,
+                           sdk_wallet_trust_anchor, alias='userA',
+                           skipverkey=True)
+
+
+@pytest.fixture(scope="module")
+def nodeSet(txnPoolNodeSet):
+    return txnPoolNodeSet
+
+
+@pytest.fixture(scope="module")
+def testNodeClass():
+    return TestNode
+
+
+@pytest.fixture(scope="module")
+def newNodeAdded(looper, nodeSet, tdir, tconf, sdk_pool_handle,
+                 sdk_wallet_trustee, allPluginsPath):
+    new_steward_wallet, new_node = sdk_node_theta_added(looper,
+                                                        nodeSet,
+                                                        tdir,
+                                                        tconf,
+                                                        sdk_pool_handle,
+                                                        sdk_wallet_trustee,
+                                                        allPluginsPath,
+                                                        node_config_helper_class=NodeConfigHelper,
+                                                        testNodeClass=TestNode,
+                                                        name='')
+    return new_steward_wallet, new_node
+
+
+@pytest.fixture(scope='module')
+def nodeIds(nodeSet):
+    return next(iter(nodeSet)).poolManager.nodeIds
