@@ -9,13 +9,14 @@ from typing import List, Tuple
 
 import base58
 import dateutil.tz
+from plenum.bls.bls_crypto_factory import create_default_bls_crypto_factory
 from plenum.test.node_catchup.helper import waitNodeDataEquality, ensure_all_nodes_have_same_data
 
 from plenum.common.keygen_utils import init_bls_keys
 
 from indy.ledger import build_pool_upgrade_request
-from plenum.common.constants import TXN_TYPE, DATA, VERSION, FORCE
-from plenum.common.types import f
+from plenum.common.constants import DATA, VERSION, FORCE
+from plenum.common.txn_util import get_type, get_payload_data, get_from
 from plenum.common.util import randomString, hexToFriendly
 from plenum.test import waits as plenumWaits
 from plenum.test.helper import sdk_get_and_check_replies
@@ -25,7 +26,7 @@ from stp_core.common.log import getlogger
 from stp_core.loop.eventually import eventually
 
 from indy_common.constants import NODE_UPGRADE, ACTION, \
-    UPGRADE_MESSAGE, MESSAGE_TYPE
+    UPGRADE_MESSAGE, MESSAGE_TYPE, APP_NAME
 from indy_common.config import controlServiceHost, controlServicePort
 from indy_node.server.upgrade_log import UpgradeLog
 from indy_node.server.upgrader import Upgrader
@@ -63,10 +64,16 @@ def get_req_from_update(looper, did, nup):
                                    json.dumps(nup['schedule']) if 'schedule' in nup else None,
                                    nup['justification'] if 'justification' in nup else 'null',
                                    nup['reinstall'] if 'reinstall' in nup else None,
-                                   nup[FORCE] if FORCE in nup else None))
+                                   nup[FORCE] if FORCE in nup else None,
+                                   nup.get('package', None)))
     return req
 
 
+def clear_aq_stash(nodes):
+    for node in nodes:
+        node.upgrader.aqStash.clear()
+
+
 def checkUpgradeScheduled(nodes, version, schedule=None):
     for node in nodes:
         assert len(node.upgrader.aqStash) == 1
@@ -103,8 +110,8 @@ def lowerVersion(v):
     return '.'.join(parts)
 
 
-def bumpedVersion():
-    v = codeVersion()
+def bumpedVersion(ver=None):
+    v = ver or codeVersion()
     return bumpVersion(v)
 
 
@@ -139,21 +146,21 @@ class NodeControlToolExecutor:
         logger.debug("NCTProcess must be stopped, with pid: {}".format(self.p.pid))
 
 
-def composeUpgradeMessage(version):
-    return (json.dumps({"version": version, MESSAGE_TYPE: UPGRADE_MESSAGE})).encode()
+def composeUpgradeMessage(version, pkg_name: str = APP_NAME):
+    return (json.dumps({"version": version, "pkg_name": pkg_name, MESSAGE_TYPE: UPGRADE_MESSAGE})).encode()
 
 
-def sendUpgradeMessage(version):
+def sendUpgradeMessage(version, pkg_name: str = APP_NAME):
     sock = socket.create_connection(
         (controlServiceHost, controlServicePort))
-    sock.sendall(composeUpgradeMessage(version))
+    sock.sendall(composeUpgradeMessage(version, pkg_name=pkg_name))
     sock.close()
 
 
 def nodeControlGeneralMonkeypatching(tool, monkeypatch, tdir, stdout):
     ret = type("", (), {})()
     ret.returncode = 0
-    ret.stdout = stdout
+    ret.stdout = stdout if isinstance(stdout, bytes) else stdout.encode()
     tool.base_dir = tdir
     tool.indy_dir = os.path.join(tool.base_dir, '.indy')
     tool.tmp_dir = os.path.join(tool.base_dir, '.indy_tmp')
@@ -170,15 +177,15 @@ def get_valid_code_hash():
 
 
 def populate_log_with_upgrade_events(
-        pool_txn_node_names, tdir, tconf, version: Tuple[str, str, str]):
+        pool_txn_node_names, tdir, tconf, version: Tuple[str, str, str], pkg_name: str = APP_NAME):
     for nm in pool_txn_node_names:
         config_helper = NodeConfigHelper(nm, tconf, chroot=tdir)
         ledger_dir = config_helper.ledger_dir
         os.makedirs(ledger_dir)
         log = UpgradeLog(os.path.join(ledger_dir, tconf.upgradeLogFile))
         when = datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc())
-        log.appendScheduled(when, version, randomString(10))
-        log.appendStarted(when, version, randomString(10))
+        log.appendScheduled(when, version, randomString(10), pkg_name)
+        log.appendStarted(when, version, randomString(10), pkg_name)
 
 
 def check_node_sent_acknowledges_upgrade(
@@ -225,6 +232,13 @@ def check_node_do_not_sent_acknowledges_upgrade(
                                node_ids=node_ids)
 
 
+def clear_config_ledger(node_set):
+    for node in node_set:
+        node.configLedger.reset_uncommitted()
+        node.configLedger._transactionLog.reset()
+        node.configLedger.tree.reset()
+
+
 def check_ledger_after_upgrade(
         node_set,
         allowed_actions,
@@ -234,19 +248,20 @@ def check_ledger_after_upgrade(
         node_ids=None):
     versions = set()
     for node in node_set:
-        # print(len(node.configLedger))
+        print(len(node.configLedger))
         assert len(node.configLedger) == ledger_size
         ids = set()
         for _, txn in node.configLedger.getAllTxn():
-            type = txn[TXN_TYPE]
+            type = get_type(txn)
             assert type in allowed_txn_types
-            data = txn
+            txn_data = get_payload_data(txn)
+            data = txn_data
             if type == NODE_UPGRADE:
-                data = txn[DATA]
+                data = txn_data[DATA]
 
             assert data[ACTION]
             assert data[ACTION] in allowed_actions
-            ids.add(txn[f.IDENTIFIER.nm])
+            ids.add(get_from(txn))
 
             assert data[VERSION]
             versions.add(data[VERSION])
@@ -258,12 +273,13 @@ def check_ledger_after_upgrade(
     assert list(versions)[0] == expected_version
 
 
-def check_no_loop(nodeSet, event):
+def check_no_loop(nodeSet, event, pkg_name: str = APP_NAME):
     for node in nodeSet:
         # mimicking upgrade start
         node.upgrader._actionLog.appendStarted(datetime.utcnow().replace(tzinfo=dateutil.tz.tzutc()),
                                                node.upgrader.scheduledAction[0],
-                                               node.upgrader.scheduledAction[2])
+                                               node.upgrader.scheduledAction[2],
+                                               pkg_name)
         node.notify_upgrade_start()
         # mimicking upgrader's initialization after restart
         node.upgrader.process_action_log_for_first_run()
@@ -281,11 +297,14 @@ def sdk_change_bls_key(looper, txnPoolNodeSet,
                        sdk_pool_handle,
                        sdk_wallet_steward,
                        add_wrong=False,
-                       new_bls=None):
-    new_blspk = init_bls_keys(node.keys_dir, node.name)
-    key_in_txn = new_bls or new_blspk \
-        if not add_wrong \
-        else base58.b58encode(randomString(128).encode())
+                       new_bls=None,
+                       new_key_proof=None):
+    if add_wrong:
+        _, new_blspk, key_proof = create_default_bls_crypto_factory().generate_bls_keys()
+    else:
+        new_blspk, key_proof = init_bls_keys(node.keys_dir, node.name)
+    key_in_txn = new_bls or new_blspk
+    bls_key_proof = new_key_proof or key_proof
     node_dest = hexToFriendly(node.nodestack.verhex)
     sdk_send_update_node(looper, sdk_wallet_steward,
                          sdk_pool_handle,
@@ -293,7 +312,8 @@ def sdk_change_bls_key(looper, txnPoolNodeSet,
                          None, None,
                          None, None,
                          bls_key=key_in_txn,
-                         services=None)
+                         services=None,
+                         key_proof=bls_key_proof)
     poolSetExceptOne = list(txnPoolNodeSet)
     poolSetExceptOne.remove(node)
     waitNodeDataEquality(looper, node, *poolSetExceptOne)
@@ -302,3 +322,15 @@ def sdk_change_bls_key(looper, txnPoolNodeSet,
                     alias=randomString(5))
     ensure_all_nodes_have_same_data(looper, txnPoolNodeSet)
     return new_blspk
+
+
+def count_action_log_entries(upg_log, func):
+    ret = 0
+    for upg_rec in upg_log:
+        if func(upg_rec):
+            ret += 1
+    return ret
+
+
+def count_action_log_package(upg_log, pkg_name):
+    return count_action_log_entries(upg_log, lambda entry: entry[5] == pkg_name)
