@@ -6,7 +6,7 @@ from plenum.common.constants import VERSION
 from indy_common.constants import FORCE
 
 from indy_node.test.upgrade.helper import bumpedVersion, checkUpgradeScheduled, \
-    check_no_loop, sdk_ensure_upgrade_sent
+    check_no_loop, sdk_ensure_upgrade_sent, clear_aq_stash
 from indy_node.server.upgrade_log import UpgradeLog
 
 whitelist = ['Failed to upgrade node',
@@ -17,11 +17,13 @@ whitelist = ['Failed to upgrade node',
 def test_upgrade_does_not_get_into_loop_force_if_failed(
         looper, tconf, nodeSet, validUpgrade, sdk_pool_handle,
         sdk_wallet_trustee, monkeypatch):
-    new_version = bumpedVersion()
+    new_version = bumpedVersion(validUpgrade['version'])
     upgr1 = deepcopy(validUpgrade)
     upgr1[VERSION] = new_version
     upgr1[FORCE] = True
 
+    clear_aq_stash(nodeSet)
+
     # An upgrade scheduled, it should pass
     sdk_ensure_upgrade_sent(looper, sdk_pool_handle, sdk_wallet_trustee, upgr1)
     looper.run(
